<?php
/**
 * Main file processor class
 *
 * @package CSVImproved
 * @author Roland Dalmulder
 * @link http://www.csvimproved.com
 * @copyright Copyright (C) 2006 - 2009 RolandD Cyber Produksi
 * @version $Id: csvi_class_file.php 903 2009-05-14 15:47:24Z Roland $
 */

defined( '_JEXEC' ) or die( 'Direct Access to this location is not allowed.' );
 
/** 
 * CsviFile class 
 * 
 * The CsviFile class handles all file operations regarding reading CSV and XLS
 * files. XLS reading is supported through the ExcelReader class.
 *
 * @package CSVImproved
 * @see Spreadsheet_Excel_Reader::read()
 */
class CsviFile {
	
	/** @var array Contains allowed extensions for uploaded files */
	var $suffixes = array();
	
	/** @var array Contains allowed mimetypes for uploaded files */
	var $mimetypes = array();
	
	/** @var string Contains the name of the uploaded file */
	var $filename = '';
	
	/** @var string Contains the extension of the uploaded file */
	var $extension = '';
	
	/** @var bool Contains the value whether or not the file uses
	 * an extension that is allowed.
	 *
	 * @see $suffixes
	 */
	var $valid_extension = false;
	
	/** @var bool Filepointer used when opening files */
	var $fp = false;
	
	/** @var integer Internal line pointer */
	var $linepointer = 2;
	
	/** @var array Contains the data that is read from file */
	var $data = null;
	
	/**
	 * Controls the reading of a file
	 *
	 * Determines on extension what file reading method to use.
	 *
	 * @todo Create valid checks for $fp
	 * @see $fp
	 */
	function __construct() {
		/* Load the necessary libraries */
	}
	
	/**
	 * Set the settings to work with
	 *
	 * @see $suffixes
	 * @see $mimetypes
	 * @see $data
	 */
	function ProcessFile($filename) {
		$this->filename = $filename;
		$this->fp = fopen($this->filename,"r");
	}
	
	/**
	 * Read the next line
	 *
	 * Reads the next line of the uploaded file
	 * In case of csv, it reads the next line from the file
	 * In case of xls, it returns the next entry in the array because the whole 
	 * file is already read
	 *
	 * @return array with the line of data read
	 */
	function ReadNextLine($field_delimiter, $text_enclosure, $enc) {
	
				$newdata = array();
				setlocale(LC_CTYPE, "ru_RU");
				/* Set the delimiter */
				if (strtolower($field_delimiter) == 't') $field_delimiter = "\t";
				
				if (!empty($text_enclosure)) $csvdata = fgetcsv($this->fp, 16384, $field_delimiter, $text_enclosure);
				else $csvdata = fgetcsv($this->fp, 16384, $field_delimiter);
                                				
				if ($csvdata) {
					foreach ($csvdata as $key => $value) {
						if ($enc==1)
							$newdata[$key] = iconv('CP1251','UTF-8',$value);
						else if ($enc==2)
							$newdata[$key] = iconv('UTF-8','CP1251',$value);
						else
							$newdata[$key] = $value;
					}
					return $newdata;
				}
				else {
					$this->CloseFile();
					return false;
				}
	}
	
	/**
	 * Close the file
	 *
	 * Closes the csv file pointer
	 *
	 * @see OpenFile()
	 */
	function CloseFile() {
		switch ($this->extension) {
			case 'csv':
				/* Close csv file */
				fclose ($this->fp);
				break;
		}
	}
	
	
	/**
	 * Checks if the uploaded file has a BOM
	 *
	 * If the uploaded file has a BOM, remove it since it only causes
	 * problems on import.
	 *
	 * @see ReadNextLine()
	 * @param &$data The string to check for a BOM
	 * @return string return the cleaned string
	 */
	function CheckBom(&$data) {
		/* Check the first three characters */
		if (strlen($data) > 3) {
			if (ord($data{0}) == 239 && ord($data{1}) == 187 && ord($data{2}) == 191) {
				return substr($data, 3, strlen($data));
			}
			else return $data;
		}
		else return $data;
	}
	
	/**
	 * Convert an XML node into an array
	 *
	 * @param &$data object XML node to convert
	 * @return array XML node as an array
	 */
	function ToArray($data, $firstline) {
		$newdata = array();
		$counter = 1;
		foreach($data as $name => $value) {
			if ($firstline) $newdata[$counter] = $name;
			else $newdata[$counter] = $value;
			$counter++;
		}
		return $newdata;
	}
}
?>